#pragma once

#include <RcppArmadillo.h>

inline arma::vec logfact(arma::mat y) {
    y.replace(0., 1.);
    return sum(y % log(y) - y + log(8 * pow(y, 3) + 4 * pow(y, 2) + y + 1. / 30.) / 6. + std::log(M_PI) / 2., 1);
}

inline arma::mat logfact_mat(arma::mat y) {
    y.replace(0., 1.);
    return y % log(y) - y + log(8 * pow(y, 3) + 4 * pow(y, 2) + y + 1. / 30.) / 6. + std::log(M_PI) / 2.;
}

inline arma::vec ki(arma::mat y) {
    arma::uword p = y.n_cols;
    return -logfact(std::move(y)) + 0.5 * double(p) ;
}

inline arma::mat logistic(arma::mat M) {
  return arma::trunc_exp(M) % pow(1. + arma::trunc_exp(M), -1) ;
}

inline arma::mat logit(arma::mat M) {
  return arma::trunc_log(M) - arma::trunc_log(1 - M) ;
}
